/*!\file:  LaunchThread.cpp
 * \brief  launch thread in a generic way, covering single and multi-threaded cases
 * This routine attempts to simplify management of multi-threading. When multi-threadeing 
 * is not requested (serial run), LaunchThread will just call the function (provided in argument 
 * list), nothing fancy there.  If multi-threading is requested, LaunchThread will launch the 
 * function on multiple threads (num_threads of them), and provide these functions with the 
 * local data they need (folded in the "gate" structure) + the thread id + the number of threads 
 * All this info is collected in the pthread_handle structure. 
 */ 

#ifdef HAVE_CONFIG_H
	#include "config.h"
#else
#error "Cannot compile with HAVE_CONFIG_H symbol! run configure first!"
#endif

#ifdef _MULTITHREADING_
#include <pthread.h>
#endif

#include "./issm_threads.h"
#include "../Alloc/alloc.h"
#include "../Exceptions/exceptions.h"

#undef __FUNCT__ 
#define __FUNCT__ "LaunchThread"

void LaunchThread(void* function(void*), void* gate,int num_threads){

	#ifdef _MULTITHREADING_
	int i;
	int* status=NULL;

	pthread_t* threads=NULL;
	pthread_handle* handles=NULL;
	
	/*dynamically allocate: */
	threads=(pthread_t*)xmalloc(num_threads*sizeof(pthread_t));
	handles=(pthread_handle*)xmalloc(num_threads*sizeof(pthread_handle));

	for(i=0;i<num_threads;i++){
		handles[i].gate=gate;
		handles[i].id=i;
		handles[i].num=num_threads;
	}
	
	for(i=0;i<num_threads;i++){

		if(pthread_create(threads+i,NULL,function,(void*)(handles+i))){
			throw ErrorException(__FUNCT__," pthread_create error");
		}
	}
	for(i=0;i<num_threads;i++){
		if(pthread_join(threads[i],(void**)&status)){
			throw ErrorException(__FUNCT__," pthread_join error");
		}
	}
	
	/*Free ressources:*/
	xfree((void**)&threads);
	xfree((void**)&handles);

	#else
	function(&gate);
	#endif

}
